/*
Copyright (c) 2019-2022 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package dpdknet

import (
	"context"
	"strconv"

	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	dpdk "repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/dpdknet"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func utilityDPDKNetCheckPresence(ctx context.Context, d *schema.ResourceData, m interface{}) (*dpdk.RecordDPDKNet, error) {
	c := m.(*controller.ControllerCfg)
	req := dpdk.GetRequest{}

	if d.Get("dpdk_id") != nil {
		if d.Get("dpdk_id").(int) == 0 {
			id, _ := strconv.ParseUint(d.Id(), 10, 64)
			req.DPDKID = id
		} else {
			req.DPDKID = uint64(d.Get("dpdk_id").(int))
		}
	} else {
		id, _ := strconv.ParseUint(d.Id(), 10, 64)
		req.DPDKID = id
	}

	log.Debugf("utilityDPDKCheckPresence: get DPDK network")
	dpdk, err := c.CloudBroker().DPDKNet().Get(ctx, req)
	if err != nil {
		return nil, err
	}

	return dpdk, nil
}

func utilityDPDKNetEnabled(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	dpdkID, _ := strconv.ParseUint(d.Id(), 10, 64)
	enabled := d.Get("enabled").(bool)

	if enabled {
		req := dpdk.EnableRequest{
			DPDKID: dpdkID,
		}

		if _, err := c.CloudBroker().DPDKNet().Enable(ctx, req); err != nil {
			return err
		}
	} else {
		req := dpdk.DisableRequest{
			DPDKID: dpdkID,
		}

		if _, err := c.CloudBroker().DPDKNet().Disable(ctx, req); err != nil {
			return err
		}
	}
	log.Debugf("resourceDPDKNetUpdate: enable=%v DPDK Network ID %s after completing its resource configuration", enabled, d.Id())

	return nil
}

func utilityDPDKNetUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	dpdkID, _ := strconv.ParseUint(d.Id(), 10, 64)

	req := dpdk.UpdateRequest{
		DPDKID: dpdkID,
	}

	if d.HasChange("name") {
		req.Name = d.Get("name").(string)
	}

	if d.HasChange("desc") {
		req.Description = d.Get("desc").(string)
	}

	if d.HasChange("vlan_id") {
		req.VlanID = uint64(d.Get("vlan_id").(int))
	}

	if d.HasChange("ovs_bridge") {
		req.OVSBridge = d.Get("ovs_bridge").(string)
	}

	if d.HasChange("account_access") {
		if accountAccess, ok := d.GetOk("account_access"); ok {
			IDs := accountAccess.([]interface{})
			for _, ID := range IDs {
				req.AccountAccess = append(req.AccountAccess, uint64(ID.(int)))
			}
		}
	}

	if d.HasChange("rg_access") {
		if rgAccess, ok := d.GetOk("rg_access"); ok {
			IDs := rgAccess.([]interface{})
			for _, ID := range IDs {
				req.RGAccess = append(req.RGAccess, uint64(ID.(int)))
			}
		}
	}

	if d.Get("enabled") == true {
		req := dpdk.DisableRequest{
			DPDKID: dpdkID,
		}

		if _, err := c.CloudBroker().DPDKNet().Disable(ctx, req); err != nil {
			return err
		}
	}

	if _, err := c.CloudBroker().DPDKNet().Update(ctx, req); err != nil {
		return err
	}

	if d.Get("enabled") == true {
		req := dpdk.EnableRequest{
			DPDKID: dpdkID,
		}

		if _, err := c.CloudBroker().DPDKNet().Enable(ctx, req); err != nil {
			return err
		}
	}

	return nil
}
