/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>
Kasim Baybikov, <kmbaybikov@basistech.ru>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package lb

import (
	"context"
	"strconv"
	"strings"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/lb"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
)

func resourceLBBackendServerCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBBackendServerCreate: call for lb_id %d, backend_name %s, server_name %s",
		d.Get("lb_id").(int),
		d.Get("backend_name").(string),
		d.Get("name").(string))
	c := m.(*controller.ControllerCfg)

	if diags := checkParamsExistenceLb(ctx, d, c); diags != nil {
		return diags
	}

	req := lb.BackendServerAddRequest{
		BackendName: d.Get("backend_name").(string),
		ServerName:  d.Get("name").(string),
		Address:     d.Get("address").(string),
		LBID:        uint64(d.Get("lb_id").(int)),
		Port:        uint64(d.Get("port").(int)),
	}

	if check, ok := d.GetOk("check"); ok {
		req.Check = check.(string)
	}
	if inter, ok := d.GetOk("inter"); ok {
		req.Inter = uint64(inter.(int))
	}
	if downinter, ok := d.GetOk("downinter"); ok {
		req.DownInter = uint64(downinter.(int))
	}
	if rise, ok := d.GetOk("rise"); ok {
		req.Rise = uint64(rise.(int))
	}
	if fall, ok := d.GetOk("fall"); ok {
		req.Fall = uint64(fall.(int))
	}
	if slowstart, ok := d.GetOk("slowstart"); ok {
		req.SlowStart = uint64(slowstart.(int))
	}
	if maxconn, ok := d.GetOk("maxconn"); ok {
		req.MaxConn = uint64(maxconn.(int))
	}
	if maxqueue, ok := d.GetOk("maxqueue"); ok {
		req.MaxQueue = uint64(maxqueue.(int))
	}
	if weight, ok := d.GetOk("weight"); ok {
		req.Weight = uint64(weight.(int))
	}

	_, err := c.CloudBroker().LB().BackendServerAdd(ctx, req)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	d.SetId(strconv.Itoa(d.Get("lb_id").(int)) + "#" + d.Get("backend_name").(string) + "#" + d.Get("name").(string))

	return resourceLBBackendServerRead(ctx, d, m)
}

func resourceLBBackendServerRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBBackendServerRead: call for id %s", d.Id())

	s, err := utilityLBBackendServerCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	lbId, _ := strconv.ParseInt(strings.Split(d.Id(), "#")[0], 10, 32)
	backendName := strings.Split(d.Id(), "#")[1]

	flattenResourceLBBackendServer(d, s, lbId, backendName)

	return nil
}

func resourceLBBackendServerDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBBackendServerDelete: call for id %s", d.Id())

	_, err := utilityLBBackendServerCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)
	req := lb.BackendServerDeleteRequest{
		LBID:        uint64(d.Get("lb_id").(int)),
		BackendName: d.Get("backend_name").(string),
		ServerName:  d.Get("name").(string),
	}

	_, err = c.CloudBroker().LB().BackendServerDelete(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}
	d.SetId("")

	return nil
}

func resourceLBBackendServerUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceLBBackendServerEdit: call for id %s", d.Id())
	c := m.(*controller.ControllerCfg)

	if diags := checkParamsExistenceLb(ctx, d, c); diags != nil {
		return diags
	}

	req := lb.BackendServerUpdateRequest{
		BackendName: d.Get("backend_name").(string),
		LBID:        uint64(d.Get("lb_id").(int)),
		ServerName:  d.Get("name").(string),
		Address:     d.Get("address").(string),
		Port:        uint64(d.Get("port").(int)),
	}

	if check, ok := d.GetOk("check"); ok {
		req.Check = check.(string)
	}
	if inter, ok := d.GetOk("inter"); ok {
		req.Inter = uint64(inter.(int))
	}
	if downinter, ok := d.GetOk("downinter"); ok {
		req.DownInter = uint64(downinter.(int))
	}
	if rise, ok := d.GetOk("rise"); ok {
		req.Rise = uint64(rise.(int))
	}
	if fall, ok := d.GetOk("fall"); ok {
		req.Fall = uint64(fall.(int))
	}
	if slowstart, ok := d.GetOk("slowstart"); ok {
		req.SlowStart = uint64(slowstart.(int))
	}
	if maxconn, ok := d.GetOk("maxconn"); ok {
		req.MaxConn = uint64(maxconn.(int))
	}
	if maxqueue, ok := d.GetOk("maxqueue"); ok {
		req.MaxQueue = uint64(maxqueue.(int))
	}
	if weight, ok := d.GetOk("weight"); ok {
		req.Weight = uint64(weight.(int))
	}

	_, err := c.CloudBroker().LB().BackendServerUpdate(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	return resourceLBBackendServerRead(ctx, d, m)
}

func ResourceLBBackendServer() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceLBBackendServerCreate,
		ReadContext:   resourceLBBackendServerRead,
		UpdateContext: resourceLBBackendServerUpdate,
		DeleteContext: resourceLBBackendServerDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: resourceLbBackendServerSchemaMake(),
	}
}
